#include <Alert.h>
#include <TextView.h>

#include "PPWindow.h"

#include "EqualizerPlugin.h"
#include "Preferences.h"

#define MAX_FREQUENCY	44100L
#define CHANNELS		2L

//------------------------------------------------------------------------------
// Pre-calculated filter coeficients, I'm lazy. And this is just a test plugin,
// not a full-featured one. May change in the future :-)

typedef struct {
	float a; // Alpha
	float b; // Beta
	float g; // Gamma
} iir_cfs;

iir_cfs coefs[10] = {
	{ 0.0007706594, 0.4984587,  0.9984491 }, //  31 Hz
	{ 0.001562013 , 0.496876 ,  0.9968368 }, //  62 Hz
	{ 0.003129769 , 0.4937405,  0.9935817 }, // 125 Hz
	{ 0.006182143 , 0.4876357,  0.9870087 }, // 250 Hz
	{ 0.01213592  , 0.4757282,  0.9732514 }, // 500 Hz
	{ 0.02340247  , 0.4531951,  0.9435273 }, //   1 kHz
	{ 0.04357446  , 0.4128511,  0.8760584 }, //   2 kHz
	{ 0.07625358  , 0.3474929,  0.7136286 }, //   4 kHz
	{ 0.1199464   , 0.2601072,  0.3176087 }, //   8 kHz
	{ 0.159503    , 0.180994 , -0.4435172 }, //  16 kHz
};

//------------------------------------------------------------------------------

// The first string is intended to be a short label for the type of audio this
// plugin handles.
// The second string should give a more complete description of the plugin.
MyPlugin::MyPlugin(struct PluginConstructorStruct *ConstrInfo)
	: DSPPlugin ("equalizer", "Graphical Equalizer - by Oscar Lesta")
{
	PPWindow::CreateWinSemaphore();
	ThePrefs.Init(ConstrInfo->SettingsFolder);
}

MyPlugin::~MyPlugin()
{
	PPWindow::DeleteWinSemaphore();
	PPWindow::QuitWindow();
}

bool MyPlugin::Init(long Frequency, short Channels, bool SampleIsOnly8Bits)
{
	if (Frequency != MAX_FREQUENCY || Channels != CHANNELS || SampleIsOnly8Bits)
		return false;

	for (int i = 0; i < 10; i++)
	{
		l_xn1[i] = l_xn2[i] = l_yn1[i] = l_yn2[i] = 0.0f;
		r_xn1[i] = r_xn2[i] = r_yn1[i] = r_yn2[i] = 0.0f;

		fg[i] = ThePrefs.Gain(i) / 100.0f;
	}

	return true;
}

void MyPlugin::Cleanup(void)
{

}

unsigned long MyPlugin::GetFlags()
{
	return 0;
}

bool MyPlugin::About(bool Question)
{
	if (!Question)
	{
		BAlert* theAlert = new BAlert("",
				"Graphical Equalizer\nVersion 0.2\n\nMade by Oscar Lesta\nE-mail: bipolar@softhome.net\n\n",
				"Bah!", NULL, NULL, B_WIDTH_FROM_LABEL);

		BTextView *theText = theAlert->TextView();
		if (theText)
		{
			theText->SetStylable(true);
			theText->Select(0, 19);	// Adjust this to the length of your header!

			BFont ourFont;

			theText->SetFontAndColor(be_bold_font);
			theText->GetFontAndColor(2, &ourFont, NULL);
			ourFont.SetSize(24);
			theText->SetFontAndColor(&ourFont);
		}

		theAlert->Go();
	}

	return true;
}

bool MyPlugin::Prefs(bool Question)
{
	if (!Question && PPWindow::RequestWindow())
	{
		PPWindow::CreateWindow();
	}

	return true;
}

long MyPlugin::Process(char* buff, long Size)
{
	short* in  = (short*) buff;

	float l_xn = 0, r_xn = 0;
	float l_yn[10], r_yn[10];

	UpdateGains();

	long temp = 0;

	for (long samples = Size/(sizeof(short) * 2); samples > 0; --samples)
	{
		// get the sample as a float number.
		l_xn = in[0] / (32767.0f);
		r_xn = in[1] / (32767.0f);

		// we have 10 filters...
		for (int j = 0; j < 10; j++)
		{
			l_yn[j] = 2 * (coefs[j].a * (l_xn - l_xn2[j]) - coefs[j].b * l_yn2[j] + coefs[j].g * l_yn1[j]);

			l_xn2[j] = l_xn1[j];	// x[n-1] -> x[n-2]
			l_xn1[j] = l_xn;		// x[n]   -> x[n-1]
			l_yn2[j] = l_yn1[j];	// y[n-1] -> y[n-2]
			l_yn1[j] = l_yn[j];		// y[n]   -> y[n-1]

	 		l_yn[j] += l_yn[j] * fg[j];

			r_yn[j] = 2 * (coefs[j].a * (r_xn - r_xn2[j]) - coefs[j].b * r_yn2[j] + coefs[j].g * r_yn1[j]);

			r_xn2[j] = r_xn1[j];	// x[n-1] -> x[n-2]
			r_xn1[j] = r_xn;		// x[n]   -> x[n-1]
			r_yn2[j] = r_yn1[j];	// y[n-1] -> y[n-2]
			r_yn1[j] = r_yn[j];		// y[n]   -> y[n-1]

	 		r_yn[j] += r_yn[j] * fg[j];
		}

		for (int i = 1; i < 10; i++)
		{
			l_yn[0] += l_yn[i];
			r_yn[0] += r_yn[i];
		}

		// Clamp values.
		temp = (long) (l_yn[0] * 32767);
		(temp > 32767) ? (temp = 32767) : (temp < -32767) ? (temp = -32767) : (temp);
		*in++ = (short) (temp);

		temp = (long) (r_yn[0] * 32767);
		(temp > 32767) ? (temp = 32767) : (temp < -32767) ? (temp = -32767) : (temp);
		*in++ = (short) (temp);
	}

	return Size;
}

void MyPlugin::Flush()
{
	// we don't hold internal buffers, do nothing.
}

void MyPlugin::UpdateGains()
{
	if (!ThePrefs.PrefsChanged())
		return;

	for (int i = 0; i < 10; i++)
	{
		fg[i] = ThePrefs.Gain(i) / 100.0f;
	}
};

//------------------------------------------------------------------------------
